@@ -5,12 +5,13 @@ lines.segmented<-function(x, term, bottom=TRUE, shift=TRUE, conf.level=0.95, k=5
                else {term<-x$nameUV$Z}
                }
   ss<-list(...)
+  metodo<- if(!is.null(ss$method)) ss$method else "delta"
   colore<- if(is.null(ss$col)) 1 else ss$col
   usr <- par("usr")
   h<-(usr[4]-usr[3])/abs(k)
   y<- if(bottom) usr[3]+h else usr[4]-h
-  r<- confint.segmented(object=x,parm=term,level=conf.level,rev.sgn=rev.sgn,digits=15)
-  m<-r[[term]]
+  m<- confint.segmented(object=x,parm=term,level=conf.level,rev.sgn=rev.sgn,digits=15,method=metodo)
+  #m<-r[[term]]
   #FORSE non e' necessaria
   #if(rev.sgn) m<- -m
   #ma invece serve il seguente (se length(psi)=1 e rev.sgn=T):
