@@ -1,6 +1,6 @@
 `slope` <-
-function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE,
-      digits = max(3, getOption("digits") - 3)){
+function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, APC=FALSE, .vcov=NULL,...,
+      digits = max(4, getOption("digits") - 2)){
 #--
 
         f.U<-function(nomiU, term=NULL){
@@ -19,12 +19,14 @@ function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE,
         }
 #--        
 #        if(!"segmented"%in%class(ogg)) stop("A 'segmented' model is requested")
-        if(var.diff && length(ogg$nameUV$Z)>1) {
-            var.diff<-FALSE
-            warning("var.diff set to FALSE with multiple segmented variables", call.=FALSE)
-            }
-    #se e' un "newsegmented"
 
+        #commentato il 28/05/19. vcov() fa gia' questo controllo..
+        # if(var.diff && length(ogg$nameUV$Z)>1) {
+        #     var.diff<-FALSE
+        #     warning("var.diff set to FALSE with multiple segmented variables", call.=FALSE)
+        #     }
+
+            #se e' un "newsegmented"
 #    if(!is.null(ogg$R.slope)) {
 #             covv<-old.coef.var(ogg)
 #             ogg$coefficients<- covv$b
@@ -32,14 +34,17 @@ function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE,
 #             ogg$psi<-old.psi(ogg)
 #             ogg$nameUV<-old.nomi(ogg)
 #             } else {
-             covv<-try(vcov(ogg,var.diff=var.diff), silent=TRUE)
+             covv<-try(vcov(ogg,...), silent=TRUE)
 #             }
-        
+
+        covv <- if(is.null(.vcov)) vcov(ogg, ...) else .vcov 
+        if(!all(dim(covv)==c(length(coef(ogg)), length(coef(ogg))))) stop("Incorrect dimension of cov matrix", call. = FALSE)
+             
         nomepsi<-rownames(ogg$psi) #OK
         nomeU<-ogg$nameUV$U
         nomeZ<-ogg$nameUV$Z
         if(missing(parm)) {
-          nomeZ<- ogg$nameUV[[3]]
+          nomeZ<- ogg$nameUV$Z
           if(length(rev.sgn)==1) rev.sgn<-rep(rev.sgn,length(nomeZ))
           }
              else {
@@ -70,13 +75,11 @@ function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE,
             psii<- ogg$psi[nomiPsi , "Est."] 
             nomiU<-grep(paste(".", nomeZ[i], sep="") , ogg$nameUV$U, value=TRUE)
             #cof<-coef(ogg)[nomiU]
-            id.cof.U<- match(nomiU, names(ogg$coefficients))
+            id.cof.U<- match(nomiU, names(coef(ogg))) #prima era names(ogg$coefficients)
             index[[i]]<-c(match(nomeZ[i],nomi), id.cof.U)
-            
-            
             }
         Ris<-list()   
-        digits <- max(3, getOption("digits") - 3)
+        #digits <- max(3, getOption("digits") - 3)
         rev.sgn<-rep(rev.sgn, length.out=length(nomeZ))
         
 #         transf=c("x","1")
@@ -85,7 +88,9 @@ function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE,
 #        my.f<-function(x)eval(parse(text=transf[1]))
 #        my.f.deriv<-function(x)eval(parse(text=transf[2]))
        
-          
+
+        #browser()
+                  
         for(i in 1:length(index)){
             ind<-as.numeric(na.omit(unlist(index[[i]])))
             M<-matrix(1,length(ind),length(ind))
@@ -93,8 +98,8 @@ function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE,
             cof<-coef(ogg)[ind]
             cof.out<-M%*%cof 
 
-            
-            if(class(covv)!="try-error"){
+            if(!inherits(covv, "try-error")){
+            #if(class(covv)[1]!="try-error"){ #inherits(covv, "try-error") #is(covv, "try-error")
                 cov.ok<-covv[ind,ind]
                 cov.out<-M%*%cov.ok%*%t(M)
                 se.out<-sqrt(diag(cov.out))
@@ -117,7 +122,7 @@ function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE,
             nomeT<-if("lm"%in%class(ogg)) "t value" else "z value"
             dimnames(ris)<-list(paste("slope", 1:nrow(ris), sep=""),c("Est.","St.Err.",nomeT,cin[1],cin[2]))
             if(APC) ris<-100*(exp(ris[,c(1,4,5)])-1)
-            Ris[[nomeZ[i]]]<-signif(ris,digits)
+            Ris[[nomeZ[i]]]<- signif(ris,digits)
             
                 } #end loop i
             #if(!missing(parm)){
